@@ -1,37 +1,26 @@
-# ------------------------------------------------------------------------------
+# (C) Copyright 2005-2025 Enthought, Inc., Austin, TX
+# All rights reserved.
 #
-#  Copyright (c) 2014, Enthought, Inc.
-#  All rights reserved.
+# This software is provided without warranty under the terms of the BSD
+# license included in LICENSE.txt and may be redistributed only under
+# the conditions described in the aforementioned license. The license
+# is also available online at http://www.enthought.com/licenses/BSD.txt
 #
-#  This software is provided without warranty under the terms of the BSD
-#  license included in /LICENSE.txt and may be redistributed only
-#  under the conditions described in the aforementioned license.  The license
-#  is also available online at http://www.enthought.com/licenses/BSD.txt
-#
-#  Thanks for using Enthought open source!
-#
-# ------------------------------------------------------------------------------
+# Thanks for using Enthought open source!
+
 """
 Tests for the ArrayOrNone TraitType.
 
 """
 
-from __future__ import absolute_import
-
 import unittest
 
-try:
-    import numpy
-except ImportError:
-    numpy_available = False
-else:
-    numpy_available = True
-
-from traits.api import ArrayOrNone, HasTraits, NO_COMPARE, TraitError
+from traits.api import ArrayOrNone, ComparisonMode, HasTraits, TraitError
 from traits.testing.unittest_tools import UnittestTools
+from traits.testing.optional_dependencies import numpy, requires_numpy
 
 
-if numpy_available:
+if numpy is not None:
     # Use of `ArrayOrNone` requires NumPy to be installed.
 
     class Foo(HasTraits):
@@ -43,10 +32,11 @@ if numpy_available:
 
         maybe_array_with_default = ArrayOrNone(value=[1, 2, 3])
 
-        maybe_array_no_compare = ArrayOrNone(comparison_mode=NO_COMPARE)
+        maybe_array_no_compare = ArrayOrNone(
+            comparison_mode=ComparisonMode.none)
 
 
-@unittest.skipUnless(numpy_available, "numpy not available")
+@requires_numpy
 class TestArrayOrNone(unittest.TestCase, UnittestTools):
     """
     Tests for the ArrayOrNone TraitType.
@@ -176,3 +166,19 @@ class TestArrayOrNone(unittest.TestCase, UnittestTools):
         foo = foo_bar.foo
         foo += 1729.0
         self.assertFalse((foo_bar.foo == foo_bar.bar).all())
+
+    def test_safe_casting(self):
+        class Bar(HasTraits):
+            unsafe_f32 = ArrayOrNone(dtype="float32")
+            safe_f32 = ArrayOrNone(dtype="float32", casting="safe")
+
+        f64 = numpy.array([1], dtype="float64")
+        f32 = numpy.array([1], dtype="float32")
+
+        b = Bar()
+
+        b.unsafe_f32 = f32
+        b.unsafe_f32 = f64
+        b.safe_f32 = f32
+        with self.assertRaises(TraitError):
+            b.safe_f32 = f64
