@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2025 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -23,18 +23,21 @@ from traits.has_traits import (
     HasTraits,
     observe,
     ObserverTraits,
-    SingletonHasTraits,
-    SingletonHasStrictTraits,
-    SingletonHasPrivateTraits,
 )
 from traits.ctrait import CTrait
-from traits.observation.api import trait
+from traits.observation.api import (
+    compile_str,
+    compile_expr,
+    NotifierNotFound,
+    trait,
+)
 from traits.observation.exception_handling import (
     pop_exception_handler,
     push_exception_handler,
 )
 from traits.traits import ForwardProperty, generic_trait
 from traits.trait_types import Event, Float, Instance, Int, List, Map, Str
+from traits.trait_errors import TraitError
 
 
 def _dummy_getter(self):
@@ -181,7 +184,7 @@ class TestCreateTraitsMetaDict(unittest.TestCase):
         # Given
         @observe(trait("value"), post_init=True, dispatch="ui")
         @observe("name")
-        def handler(event):
+        def handler(self, event):
             pass
 
         class_name = "MyClass"
@@ -197,13 +200,13 @@ class TestCreateTraitsMetaDict(unittest.TestCase):
             {
                 "my_listener": [
                     {
-                        "expression": [trait("name")],
+                        "graphs": compile_str("name"),
                         "post_init": False,
                         "dispatch": "same",
                         "handler_getter": getattr,
                     },
                     {
-                        "expression": [trait("value")],
+                        "graphs": compile_expr(trait("value")),
                         "post_init": True,
                         "dispatch": "ui",
                         "handler_getter": getattr,
@@ -464,6 +467,186 @@ class TestHasTraits(unittest.TestCase):
         self.assertIsNot(objs_copy[0], objs[0])
         self.assertIs(objs_copy[0], objs_copy[1])
 
+    def test_add_class_trait(self):
+        # Testing basic usage.
+        class A(HasTraits):
+            pass
+
+        A.add_class_trait("y", Str())
+
+        a = A()
+
+        self.assertEqual(a.y, "")
+
+    def test_add_class_trait_affects_existing_instances(self):
+        class A(HasTraits):
+            pass
+
+        a = A()
+
+        A.add_class_trait("y", Str())
+
+        self.assertEqual(a.y, "")
+
+    def test_add_class_trait_affects_subclasses(self):
+        class A(HasTraits):
+            pass
+
+        class B(A):
+            pass
+
+        class C(B):
+            pass
+
+        class D(B):
+            pass
+
+        A.add_class_trait("y", Str())
+        self.assertEqual(A().y, "")
+        self.assertEqual(B().y, "")
+        self.assertEqual(C().y, "")
+        self.assertEqual(D().y, "")
+
+    def test_add_class_trait_has_items_and_subclasses(self):
+        # Regression test for enthought/traits#1460
+        class A(HasTraits):
+            pass
+
+        class B(A):
+            pass
+
+        class C(B):
+            pass
+
+        # Code branch for traits with items.
+        A.add_class_trait("x", List(Int))
+        self.assertEqual(A().x, [])
+        self.assertEqual(B().x, [])
+        self.assertEqual(C().x, [])
+
+        # Exercise the code branch for mapped traits.
+        A.add_class_trait("y", Map({"yes": 1, "no": 0}, default_value="no"))
+        self.assertEqual(A().y, "no")
+        self.assertEqual(B().y, "no")
+        self.assertEqual(C().y, "no")
+
+    def test_add_class_trait_add_prefix_traits(self):
+
+        class A(HasTraits):
+            pass
+
+        A.add_class_trait("abc_", Str())
+        A.add_class_trait("abc_def_", Int())
+
+        a = A()
+        self.assertEqual(a.abc_def_g, 0)
+        self.assertEqual(a.abc_z, "")
+
+    def test_add_class_trait_when_trait_already_exists(self):
+
+        class A(HasTraits):
+            foo = Int()
+
+        with self.assertRaises(TraitError):
+            A.add_class_trait("foo", List())
+
+        self.assertEqual(A().foo, 0)
+        with self.assertRaises(AttributeError):
+            A().foo_items
+
+    def test_add_class_trait_when_trait_already_exists_in_subclass(self):
+        class A(HasTraits):
+            pass
+
+        class B(A):
+            foo = Int()
+
+        A.add_class_trait("foo", Str())
+
+        self.assertEqual(A().foo, "")
+        self.assertEqual(B().foo, 0)
+
+    def test_traits_method_with_dunder_metadata(self):
+        # Regression test for enthought/envisage#430
+        class A(HasTraits):
+            foo = Int(__extension_point__=True)
+            bar = Int(__extension_point__=False)
+            baz = Int()
+
+        a = A(foo=3, bar=4, baz=5)
+
+        self.assertEqual(
+            a.traits(__extension_point__=True),
+            {"foo": a.trait("foo")},
+        )
+
+        self.assertEqual(
+            A.class_traits(__extension_point__=True),
+            {"foo": A.class_traits()["foo"]},
+        )
+
+    def test_decorated_changed_method(self):
+        # xref: enthought/traits#527
+        # Traits should ignore the _changed magic naming.
+
+        events = []
+
+        class A(HasTraits):
+            foo = Int()
+
+            @on_trait_change("foo")
+            def _foo_changed(self, obj, name, old, new):
+                events.append((obj, name, old, new))
+
+        a = A()
+        a.foo = 23
+        self.assertEqual(
+            events,
+            [(a, "foo", 0, 23)],
+        )
+
+    def test_observed_changed_method(self):
+        events = []
+
+        class A(HasTraits):
+            foo = Int()
+
+            @observe("foo")
+            def _foo_changed(self, event):
+                events.append(event)
+
+        a = A()
+        a.foo = 23
+        self.assertEqual(len(events), 1)
+        event = events[0]
+        self.assertEqual(event.object, a)
+        self.assertEqual(event.name, "foo")
+        self.assertEqual(event.old, 0)
+        self.assertEqual(event.new, 23)
+
+    def test_decorated_changed_method_subclass(self):
+        # xref: enthought/traits#527
+        # Traits should ignore the _changed magic naming.
+
+        events = []
+
+        class A(HasTraits):
+            foo = Int()
+
+            @on_trait_change("foo")
+            def _foo_changed(self, obj, name, old, new):
+                events.append((obj, name, old, new))
+
+        class B(A):
+            pass
+
+        a = B()
+        a.foo = 23
+        self.assertEqual(
+            events,
+            [(a, "foo", 0, 23)],
+        )
+
 
 class TestObjectNotifiers(unittest.TestCase):
     """ Test calling object notifiers. """
@@ -574,27 +757,6 @@ class TestCallNotifiers(unittest.TestCase):
         self.assertEqual(side_effects, ["object1", "object2"])
 
 
-class TestDeprecatedHasTraits(unittest.TestCase):
-    def test_deprecated(self):
-        class TestSingletonHasTraits(SingletonHasTraits):
-            pass
-
-        class TestSingletonHasStrictTraits(SingletonHasStrictTraits):
-            pass
-
-        class TestSingletonHasPrivateTraits(SingletonHasPrivateTraits):
-            pass
-
-        with self.assertWarns(DeprecationWarning):
-            TestSingletonHasTraits()
-
-        with self.assertWarns(DeprecationWarning):
-            TestSingletonHasStrictTraits()
-
-        with self.assertWarns(DeprecationWarning):
-            TestSingletonHasPrivateTraits()
-
-
 class MappedWithDefault(HasTraits):
 
     married = Map({"yes": 1, "yeah": 1, "no": 0, "nah": 0})
@@ -678,6 +840,13 @@ class TestHasTraitsObserveHook(unittest.TestCase):
         person.age += 1
         self.assertEqual(len(events), 1)  # unchanged
 
+    def test_observe_method_remove_nonexistent_handler(self):
+        events = []
+        person = Person()
+
+        with self.assertRaises(NotifierNotFound):
+            person.observe(events.append, "age", remove=True)
+
     def test_observe_dispatch_ui(self):
         # Test to ensure "ui" is one of the allowed value
         # Not testing the actual effect as it requires GUI event loop
@@ -704,7 +873,7 @@ class TestHasTraitsObserveHook(unittest.TestCase):
         self.assertEqual(len(instance.events), 1)
 
     def test_observer_overridden(self):
-        # The handler is overriden, no change event should be registered.
+        # The handler is overridden, no change event should be registered.
         class BaseClass(HasTraits):
             events = List()
 
@@ -712,11 +881,11 @@ class TestHasTraitsObserveHook(unittest.TestCase):
             def handler(self, event):
                 self.events.append(event)
 
-        class SubclassOverriden(BaseClass):
+        class SubclassOverridden(BaseClass):
             value = Int()
             handler = None
 
-        instance = SubclassOverriden()
+        instance = SubclassOverridden()
         instance.value += 1
         self.assertEqual(len(instance.events), 0)
 
