@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2025 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -253,6 +253,6 @@ class ListEventTestCase(unittest.TestCase):
 
         self.assertEqual(len(foo.l_events), 1)
         event, = foo.l_events
-        self.assertEqual(event.index, slice(None, None, 2))
+        self.assertEqual(event.index, slice(0, 3, 2))
         self.assertEqual(event.added, [3, 4])
         self.assertEqual(event.removed, [1, 3])
