@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2025 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -31,7 +31,3 @@ class TestCaseNoneTrait(unittest.TestCase):
         with self.assertRaises(ValueError):
             class TestClass(HasTraits):
                 none_trait = _NoneTrait(default_value=[])
-
-
-if __name__ == '__main__':
-    unittest.main()
