@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2022 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2025 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -407,9 +407,12 @@ class TestHasTraitsPropertyObserves(unittest.TestCase):
     def test_property_default_initializer_with_value_in_init(self):
         # With "depends_on", instantiation will fail.
         # enthought/traits#709
-        with self.assertRaises(AttributeError):
-            ClassWithPropertyDependsOnInit(
-                info_without_default=PersonInfo(age=30))
+        with self.assertWarnsRegex(
+            UserWarning, "default value resolution raised an AttributeError"
+        ):
+            with self.assertRaises(AttributeError):
+                ClassWithPropertyDependsOnInit(
+                    info_without_default=PersonInfo(age=30))
 
         # With "observe", initialization works.
         instance = ClassWithPropertyObservesInit(
