@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2022 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2025 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -37,31 +37,10 @@ class TestTraitGet(UnittestTools, unittest.TestCase):
         values = obj.trait_get("string", "integer")
         self.assertEqual(values, {"string": "foo", "integer": 0})
 
-    def test_trait_set_deprecated(self):
-        obj = TraitsObject()
-
-        with self.assertNotDeprecated():
-            obj.trait_set(integer=1)
-
-        with self.assertDeprecated():
-            obj.set(string="foo")
-
-        self.assertEqual(obj.string, "foo")
-        self.assertEqual(obj.integer, 1)
-
-    def test_trait_get_deprecated(self):
-        obj = TraitsObject()
-        obj.string = "foo"
         obj.integer = 1
-
-        with self.assertNotDeprecated():
-            values = obj.trait_get("integer")
+        values = obj.trait_get("integer")
         self.assertEqual(values, {"integer": 1})
 
-        with self.assertDeprecated():
-            values = obj.get("string")
-        self.assertEqual(values, {"string": "foo"})
-
     def test_trait_set_quiet(self):
         obj = TraitsObject()
         obj.string = "foo"
