@@ -1,46 +1,35 @@
-# ------------------------------------------------------------------------------
-# Copyright (c) 2005, Enthought, Inc.
+# (C) Copyright 2005-2025 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
-# license included in enthought/LICENSE.txt and may be redistributed only
-# under the conditions described in the aforementioned license.  The license
+# license included in LICENSE.txt and may be redistributed only under
+# the conditions described in the aforementioned license. The license
 # is also available online at http://www.enthought.com/licenses/BSD.txt
-# Thanks for using Enthought open source!
 #
-# Author: David C. Morrill Date: 03/20/2003 Description: Unit Test Case for the
-# Traits Package
-# ------------------------------------------------------------------------------
+# Thanks for using Enthought open source!
 
 #  Imports
-from __future__ import absolute_import, print_function
 
 import unittest
 
-import six
-
 from traits.api import (
     Any,
     Bytes,
     CBytes,
     CFloat,
     CInt,
-    CLong,
+    ComparisonMode,
     Delegate,
     Float,
     HasTraits,
     Instance,
     Int,
     List,
-    Long,
+    Range,
     Str,
     This,
     Trait,
     TraitError,
-    TraitList,
-    TraitPrefixList,
-    TraitPrefixMap,
-    TraitRange,
     Tuple,
     pop_exception_handler,
     push_exception_handler,
@@ -48,11 +37,6 @@ from traits.api import (
 
 #  Base unit test classes:
 
-if six.PY2:
-    LONG_TYPE = long
-else:
-    LONG_TYPE = int
-
 
 class BaseTest(object):
     def assign(self, value):
@@ -93,8 +77,7 @@ class test_base2(unittest.TestCase):
         list[index1:index2:step] = value
 
     # This avoids using a method name that contains 'test' so that this is not
-    # called by the tester directly, as nose looks for all tests, regardless of
-    # the handler at the bottom of this file.
+    # called by the tester directly.
     def check_values(
         self,
         name,
@@ -105,41 +88,27 @@ class test_base2(unittest.TestCase):
         mapped_values=None,
     ):
         obj = self.obj
-        try:
-            # Make sure the default value is correct:
-            msg = "default value"
-            value = default_value
-            self.assertEqual(getattr(obj, name), value)
 
-            # Iterate over all legal values being tested:
-            if actual_values is None:
-                actual_values = good_values
-            msg = "legal values"
-            i = 0
-            for value in good_values:
-                setattr(obj, name, value)
-                self.assertEqual(getattr(obj, name), actual_values[i])
-                if mapped_values is not None:
-                    self.assertEqual(
-                        getattr(obj, name + "_"), mapped_values[i]
-                    )
-                i += 1
+        # Make sure the default value is correct:
+        value = default_value
+        self.assertEqual(getattr(obj, name), value)
 
-            # Iterate over all illegal values being tested:
-            msg = "illegal values"
-            for value in bad_values:
-                self.assertRaises(TraitError, setattr, obj, name, value)
-        except:
-            print(
-                "Failed while testing %s for value: %s(%s) in %s"
-                % (
-                    msg,
-                    value,
-                    value.__class__.__name__,
-                    self.__class__.__name__,
+        # Iterate over all legal values being tested:
+        if actual_values is None:
+            actual_values = good_values
+        i = 0
+        for value in good_values:
+            setattr(obj, name, value)
+            self.assertEqual(getattr(obj, name), actual_values[i])
+            if mapped_values is not None:
+                self.assertEqual(
+                    getattr(obj, name + "_"), mapped_values[i]
                 )
-            )
-            raise
+            i += 1
+
+        # Iterate over all illegal values being tested:
+        for value in bad_values:
+            self.assertRaises(TraitError, setattr, obj, name, value)
 
 
 class AnyTrait(HasTraits):
@@ -148,10 +117,11 @@ class AnyTrait(HasTraits):
 
 class AnyTraitTest(BaseTest, unittest.TestCase):
 
-    obj = AnyTrait()
+    def setUp(self):
+        self.obj = AnyTrait()
 
     _default_value = None
-    _good_values = [10.0, "ten", u"ten", [10], {"ten": 10}, (10,), None, 1j]
+    _good_values = [10.0, b"ten", "ten", [10], {"ten": 10}, (10,), None, 1j]
     _mapped_values = []
     _bad_values = []
 
@@ -166,32 +136,31 @@ class IntTrait(HasTraits):
 
 class CoercibleIntTest(AnyTraitTest):
 
-    obj = CoercibleIntTrait()
+    def setUp(self):
+        self.obj = CoercibleIntTrait()
 
     _default_value = 99
     _good_values = [
         10,
         -10,
-        LONG_TYPE(10),
-        LONG_TYPE(-10),
         10.1,
         -10.1,
         "10",
         "-10",
-        u"10",
-        u"-10",
+        b"10",
+        b"-10",
     ]
     _bad_values = [
         "10L",
         "-10L",
         "10.1",
         "-10.1",
-        u"10L",
-        u"-10L",
-        u"10.1",
-        u"-10.1",
+        b"10L",
+        b"-10L",
+        b"10.1",
+        b"-10.1",
         "ten",
-        u"ten",
+        b"ten",
         [10],
         {"ten": 10},
         (10,),
@@ -203,21 +172,19 @@ class CoercibleIntTest(AnyTraitTest):
         try:
             return int(value)
         except:
-            try:
-                return int(float(value))
-            except:
-                return int(LONG_TYPE(value))
+            return int(float(value))
 
 
 class IntTest(AnyTraitTest):
 
-    obj = IntTrait()
+    def setUp(self):
+        self.obj = IntTrait()
 
     _default_value = 99
-    _good_values = [10, -10, LONG_TYPE(10), LONG_TYPE(-10)]
+    _good_values = [10, -10]
     _bad_values = [
         "ten",
-        u"ten",
+        b"ten",
         [10],
         {"ten": 10},
         (10,),
@@ -229,14 +196,14 @@ class IntTest(AnyTraitTest):
         "-10L",
         "10.1",
         "-10.1",
-        u"10L",
-        u"-10L",
-        u"10.1",
-        u"-10.1",
+        b"10L",
+        b"-10L",
+        b"10.1",
+        b"-10.1",
         "10",
         "-10",
-        u"10",
-        u"-10",
+        b"10",
+        b"-10",
     ]
 
     try:
@@ -259,99 +226,7 @@ class IntTest(AnyTraitTest):
         try:
             return int(value)
         except:
-            try:
-                return int(float(value))
-            except:
-                return int(LONG_TYPE(value))
-
-
-class CoercibleLongTrait(HasTraits):
-    value = CLong(LONG_TYPE(99))
-
-
-class LongTrait(HasTraits):
-    value = Long(LONG_TYPE(99))
-
-
-class CoercibleLongTest(AnyTraitTest):
-
-    obj = CoercibleLongTrait()
-
-    _default_value = LONG_TYPE(99)
-    _good_values = [
-        10,
-        -10,
-        LONG_TYPE(10),
-        LONG_TYPE(-10),
-        10.1,
-        -10.1,
-        "10",
-        "-10",
-        u"10",
-        u"-10",
-    ]
-    if six.PY2:
-        _good_values.extend(["10L", "-10L", u"10L", u"-10L"])
-    _bad_values = [
-        "10.1",
-        "-10.1",
-        u"10.1",
-        u"-10.1",
-        "ten",
-        u"ten",
-        [10],
-        [LONG_TYPE(10)],
-        {"ten": 10},
-        (10,),
-        (LONG_TYPE(10),),
-        None,
-        1j,
-    ]
-
-    def coerce(self, value):
-        try:
-            return LONG_TYPE(value)
-        except:
-            return LONG_TYPE(float(value))
-
-
-class LongTest(AnyTraitTest):
-
-    obj = LongTrait()
-
-    _default_value = LONG_TYPE(99)
-    _good_values = [10, -10, LONG_TYPE(10), LONG_TYPE(-10)]
-    _bad_values = [
-        "ten",
-        u"ten",
-        [10],
-        [LONG_TYPE(10)],
-        {"ten": 10},
-        (10,),
-        (LONG_TYPE(10),),
-        None,
-        1j,
-        10.1,
-        -10.1,
-        "10",
-        "-10",
-        "10L",
-        "-10L",
-        "10.1",
-        "-10.1",
-        u"10",
-        u"-10",
-        u"10L",
-        u"-10L",
-        u"10.1",
-        u"-10.1",
-    ]
-
-    def coerce(self, value):
-        try:
-            return LONG_TYPE(value)
-        except:
-            return LONG_TYPE(float(value))
+            return int(float(value))
 
 
 class CoercibleFloatTrait(HasTraits):
@@ -363,32 +238,31 @@ class FloatTrait(HasTraits):
 
 
 class CoercibleFloatTest(AnyTraitTest):
-    obj = CoercibleFloatTrait()
+    def setUp(self):
+        self.obj = CoercibleFloatTrait()
 
     _default_value = 99.0
     _good_values = [
         10,
         -10,
-        LONG_TYPE(10),
-        LONG_TYPE(-10),
         10.1,
         -10.1,
         "10",
         "-10",
         "10.1",
         "-10.1",
-        u"10",
-        u"-10",
-        u"10.1",
-        u"-10.1",
+        b"10",
+        b"-10",
+        b"10.1",
+        b"-10.1",
     ]
     _bad_values = [
         "10L",
         "-10L",
-        u"10L",
-        u"-10L",
+        b"10L",
+        b"-10L",
         "ten",
-        u"ten",
+        b"ten",
         [10],
         {"ten": 10},
         (10,),
@@ -397,20 +271,18 @@ class CoercibleFloatTest(AnyTraitTest):
     ]
 
     def coerce(self, value):
-        try:
-            return float(value)
-        except:
-            return float(LONG_TYPE(value))
+        return float(value)
 
 
 class FloatTest(AnyTraitTest):
-    obj = FloatTrait()
+    def setUp(self):
+        self.obj = FloatTrait()
 
     _default_value = 99.0
     _good_values = [10, -10, 10.1, -10.1]
     _bad_values = [
         "ten",
-        u"ten",
+        b"ten",
         [10],
         {"ten": 10},
         (10,),
@@ -422,22 +294,16 @@ class FloatTest(AnyTraitTest):
         "-10L",
         "10.1",
         "-10.1",
-        u"10",
-        u"-10",
-        u"10L",
-        u"-10L",
-        u"10.1",
-        u"-10.1",
+        b"10",
+        b"-10",
+        b"10L",
+        b"-10L",
+        b"10.1",
+        b"-10.1",
     ]
 
-    if six.PY2:
-        _good_values.extend([LONG_TYPE(-10), LONG_TYPE(10)])
-
     def coerce(self, value):
-        try:
-            return float(value)
-        except:
-            return float(LONG_TYPE(value))
+        return float(value)
 
 
 #  Trait that can only have 'complex'(i.e. imaginary) values:
@@ -448,15 +314,13 @@ class ImaginaryValueTrait(HasTraits):
 
 
 class ImaginaryValueTest(AnyTraitTest):
-
-    obj = ImaginaryValueTrait()
+    def setUp(self):
+        self.obj = ImaginaryValueTrait()
 
     _default_value = 99.0 - 99.0j
     _good_values = [
         10,
         -10,
-        LONG_TYPE(10),
-        LONG_TYPE(-10),
         10.1,
         -10.1,
         "10",
@@ -473,13 +337,10 @@ class ImaginaryValueTest(AnyTraitTest):
         "10+10j",
         "10-10j",
     ]
-    _bad_values = [u"10L", u"-10L", "ten", [10], {"ten": 10}, (10,), None]
+    _bad_values = [b"10L", "-10L", "ten", [10], {"ten": 10}, (10,), None]
 
     def coerce(self, value):
-        try:
-            return complex(value)
-        except:
-            return complex(LONG_TYPE(value))
+        return complex(value)
 
 
 class StringTrait(HasTraits):
@@ -488,14 +349,13 @@ class StringTrait(HasTraits):
 
 class StringTest(AnyTraitTest):
 
-    obj = StringTrait()
+    def setUp(self):
+        self.obj = StringTrait()
 
     _default_value = "string"
     _good_values = [
         10,
         -10,
-        LONG_TYPE(10),
-        LONG_TYPE(-10),
         10.1,
         -10.1,
         "10",
@@ -505,7 +365,6 @@ class StringTest(AnyTraitTest):
         "10.1",
         "-10.1",
         "string",
-        u"string",
         1j,
         [10],
         ["ten"],
@@ -519,68 +378,21 @@ class StringTest(AnyTraitTest):
         return str(value)
 
 
-class UnicodeTrait(HasTraits):
-    value = Trait(u"unicode")
-
-
-class UnicodeTest(StringTest):
-
-    obj = UnicodeTrait()
-
-    _default_value = u"unicode"
-    _good_values = [
-        10,
-        -10,
-        LONG_TYPE(10),
-        LONG_TYPE(-10),
-        10.1,
-        -10.1,
-        "10",
-        "-10",
-        "10L",
-        "-10L",
-        "10.1",
-        "-10.1",
-        "",
-        u"",
-        "string",
-        u"string",
-        1j,
-        [10],
-        ["ten"],
-        [u"ten"],
-        {"ten": 10},
-        (10,),
-        None,
-    ]
-    _bad_values = []
-
-    def coerce(self, value):
-        return str(value)
-
-
 class BytesTrait(HasTraits):
     value = Bytes(b"bytes")
 
 
-version_dependent = ["", "string"]
-
-
 class BytesTest(StringTest):
 
-    obj = BytesTrait()
+    def setUp(self):
+        self.obj = BytesTrait()
 
     _default_value = b"bytes"
-    _good_values = [b"", b"10", b"-10"] + (
-        version_dependent if six.PY2 else []
-    )
+    _good_values = [b"", b"10", b"-10"]
     _bad_values = [
         10,
         -10,
-        LONG_TYPE(10),
         10.1,
-        u"unicode",
-        u"",
         [b""],
         [b"bytes"],
         [0],
@@ -588,7 +400,9 @@ class BytesTest(StringTest):
         (b"",),
         None,
         True,
-    ] + ([] if six.PY2 else version_dependent)
+        "",
+        "string",
+    ]
 
     def coerce(self, value):
         return bytes(value)
@@ -598,32 +412,10 @@ class CoercibleBytesTrait(HasTraits):
     value = CBytes(b"bytes")
 
 
-version_dependent = [
-    "",
-    "string",
-    u"unicode",
-    u"",
-    -10,
-    10.1,
-    [b""],
-    [b"bytes"],
-    [-10],
-    (-10,),
-    {-10: "foo"},
-    set([-10]),
-    [256],
-    (256,),
-    {256: "foo"},
-    set([256]),
-    {b"ten": b"10"},
-    (b"",),
-    None,
-]
-
-
 class CoercibleBytesTest(StringTest):
 
-    obj = CoercibleBytesTrait()
+    def setUp(self):
+        self.obj = CoercibleBytesTrait()
 
     _default_value = b"bytes"
     _good_values = [
@@ -631,29 +423,47 @@ class CoercibleBytesTest(StringTest):
         b"10",
         b"-10",
         10,
-        LONG_TYPE(10),
         [10],
         (10,),
         set([10]),
         {10: "foo"},
         True,
-    ] + (version_dependent if six.PY2 else [])
-    _bad_values = [] if six.PY2 else version_dependent
+    ]
+    _bad_values = [
+        "",
+        "string",
+        -10,
+        10.1,
+        [b""],
+        [b"bytes"],
+        [-10],
+        (-10,),
+        {-10: "foo"},
+        set([-10]),
+        [256],
+        (256,),
+        {256: "foo"},
+        set([256]),
+        {b"ten": b"10"},
+        (b"",),
+        None,
+    ]
 
     def coerce(self, value):
         return bytes(value)
 
 
 class EnumTrait(HasTraits):
-    value = Trait([1, "one", 2, "two", 3, "three", 4.4, u"four.four"])
+    value = Trait([1, "one", 2, "two", 3, "three", 4.4, "four.four"])
 
 
 class EnumTest(AnyTraitTest):
 
-    obj = EnumTrait()
+    def setUp(self):
+        self.obj = EnumTrait()
 
     _default_value = 1
-    _good_values = [1, "one", 2, "two", 3, "three", 4.4, u"four.four"]
+    _good_values = [1, "one", 2, "two", 3, "three", 4.4, "four.four"]
     _bad_values = [0, "zero", 4, None]
 
 
@@ -662,7 +472,8 @@ class MappedTrait(HasTraits):
 
 
 class MappedTest(AnyTraitTest):
-    obj = MappedTrait()
+    def setUp(self):
+        self.obj = MappedTrait()
 
     _default_value = "one"
     _good_values = ["one", "two", "three"]
@@ -670,109 +481,64 @@ class MappedTest(AnyTraitTest):
     _bad_values = ["four", 1, 2, 3, [1], (1,), {1: 1}, None]
 
 
-class PrefixListTrait(HasTraits):
-    value = Trait("one", TraitPrefixList("one", "two", "three"))
+# This test a combination of Trait, a default, a mapping and a function
+
+def str_cast_to_int(object, name, value):
+    """ A function that validates the value is a str and then converts
+    it to an int using its length.
+    """
+    if not isinstance(value, str):
+        raise TraitError("Not a string!")
+    return len(value)
 
 
-class PrefixListTest(AnyTraitTest):
-    obj = PrefixListTrait()
+class TraitWithMappingAndCallable(HasTraits):
 
-    _default_value = "one"
-    _good_values = [
-        "o",
-        "on",
-        "one",
-        "tw",
-        "two",
-        "th",
-        "thr",
-        "thre",
-        "three",
-    ]
-    _bad_values = ["t", "one ", " two", 1, None]
-
-    def coerce(self, value):
-        return {"o": "one", "on": "one", "tw": "two", "th": "three"}[value[:2]]
+    value = Trait(
+        "white",
+        {"white": 0, "red": 1, (0, 0, 0): 999},
+        str_cast_to_int,
+    )
 
 
-class PrefixMapTrait(HasTraits):
-    value = Trait("one", TraitPrefixMap({"one": 1, "two": 2, "three": 3}))
+class TestTraitWithMappingAndCallable(unittest.TestCase):
+    """ Test that demonstrates a usage of Trait where TraitMap is used but it
+    cannot be replaced with Map. The callable causes the key value to be
+    changed to match the mapped value.
 
+    e.g. this would not work:
 
-class PrefixMapTest(AnyTraitTest):
-    obj = PrefixMapTrait()
+        value = Union(
+            Map({"white": 0, "red": 1, (0,0,0): 999}),
+            NewTraitType(),
+            default_value="white",
+        )
 
-    _default_value = "one"
-    _good_values = [
-        "o",
-        "on",
-        "one",
-        "tw",
-        "two",
-        "th",
-        "thr",
-        "thre",
-        "three",
-    ]
-    _mapped_values = [1, 1, 1, 2, 2, 3, 3, 3]
-    _bad_values = ["t", "one ", " two", 1, None]
+        where NewTraitType is a subclass of TraitType with ``validate`` simply
+        calls str_cast_to_int
+    """
 
-    def coerce(self, value):
-        return {"o": "one", "on": "one", "tw": "two", "th": "three"}[value[:2]]
+    def test_trait_default(self):
+        obj = TraitWithMappingAndCallable()
 
+        # the value is not 'white' any more.
+        self.assertEqual(obj.value, 5)
+        self.assertEqual(obj.value_, 5)
 
-class IntRangeTrait(HasTraits):
-    value = Trait(3, TraitRange(2, 5))
+    def test_trait_set_value_use_callable(self):
+        obj = TraitWithMappingAndCallable(value="red")
 
+        # The value is not 'red' any more.
+        # the callable is used, not the mapping.
+        self.assertEqual(obj.value, 3)
+        self.assertEqual(obj.value_, 3)
 
-class IntRangeTest(AnyTraitTest):
+    def test_trait_set_value_use_mapping(self):
+        obj = TraitWithMappingAndCallable(value=(0, 0, 0))
 
-    obj = IntRangeTrait()
-
-    _default_value = 3
-    _good_values = [2, 3, 4, 5]
-    _bad_values = [0, 1, 6, 0.999, 6.01, "two", "0.999", "6.01", None]
-
-    def coerce(self, value):
-        try:
-            return int(value)
-        except:
-            try:
-                return int(float(value))
-            except:
-                return int(LONG_TYPE(value))
-
-
-class FloatRangeTrait(HasTraits):
-    value = Trait(3.0, TraitRange(2.0, 5.0))
-
-
-class FloatRangeTest(AnyTraitTest):
-
-    obj = FloatRangeTrait()
-
-    _default_value = 3.0
-    _good_values = [2.0, 3.0, 4.0, 5.0, 2.001, 4.999]
-    _bad_values = [
-        0,
-        1,
-        6,
-        LONG_TYPE(0),
-        LONG_TYPE(1),
-        LONG_TYPE(6),
-        1.999,
-        6.01,
-        "two",
-        "0.999",
-        "6.01",
-        None,
-    ]
-
-    def coerce(self, value):
-        try:
-            return float(value)
-        except:
-            return float(LONG_TYPE(value))
+        # Now this uses the mapping, and the value is the original one.
+        self.assertEqual(obj.value, (0, 0, 0))
+        self.assertEqual(obj.value_, 999)
 
 
 # Old style class version:
@@ -802,7 +568,8 @@ class OldInstanceTrait(HasTraits):
 
 
 class OldInstanceTest(AnyTraitTest):
-    obj = OldInstanceTrait()
+    def setUp(self):
+        self.obj = OldInstanceTrait()
 
     _default_value = otrait_test1
     _good_values = [
@@ -814,14 +581,13 @@ class OldInstanceTest(AnyTraitTest):
     ]
     _bad_values = [
         0,
-        LONG_TYPE(0),
         0.0,
         0j,
         OTraitTest1,
         OTraitTest2,
         OBadTraitTest(),
+        b"bytes",
         "string",
-        u"string",
         [otrait_test1],
         (otrait_test1,),
         {"data": otrait_test1},
@@ -853,7 +619,8 @@ class NewInstanceTrait(HasTraits):
 
 
 class NewInstanceTest(AnyTraitTest):
-    obj = NewInstanceTrait()
+    def setUp(self):
+        self.obj = NewInstanceTrait()
 
     _default_value = ntrait_test1
     _good_values = [
@@ -865,14 +632,13 @@ class NewInstanceTest(AnyTraitTest):
     ]
     _bad_values = [
         0,
-        LONG_TYPE(0),
         0.0,
         0j,
         NTraitTest1,
         NTraitTest2,
         NBadTraitTest(),
+        b"bytes",
         "string",
-        u"string",
         [ntrait_test1],
         (ntrait_test1,),
         {"data": ntrait_test1},
@@ -943,7 +709,8 @@ class OddIntegerTrait(HasTraits):
 
 
 class OddIntegerTest(AnyTraitTest):
-    obj = OddIntegerTrait()
+    def setUp(self):
+        self.obj = OddIntegerTrait()
 
     _default_value = 99
     _good_values = [
@@ -953,12 +720,6 @@ class OddIntegerTest(AnyTraitTest):
         7,
         9,
         999999999,
-        LONG_TYPE(1),
-        LONG_TYPE(3),
-        LONG_TYPE(5),
-        LONG_TYPE(7),
-        LONG_TYPE(9),
-        LONG_TYPE(999999999),
         1.0,
         3.0,
         5.0,
@@ -971,12 +732,6 @@ class OddIntegerTest(AnyTraitTest):
         -7,
         -9,
         -999999999,
-        LONG_TYPE(-1),
-        LONG_TYPE(-3),
-        LONG_TYPE(-5),
-        LONG_TYPE(-7),
-        LONG_TYPE(-9),
-        LONG_TYPE(-999999999),
         -1.0,
         -3.0,
         -5.0,
@@ -1007,13 +762,8 @@ class NotifierTraits(HasTraits):
 
 
 class NotifierTests(unittest.TestCase):
-    obj = NotifierTraits()
-
-    def __init__(self, value):
-        unittest.TestCase.__init__(self, value)
-
     def setUp(self):
-        obj = self.obj
+        obj = self.obj = NotifierTraits()
         obj.value1 = 0
         obj.value2 = 0
         obj.value1_count = 0
@@ -1179,7 +929,7 @@ class DelegateTests(unittest.TestCase):
 
 # Make a TraitCompound handler that does not have a fast_validate so we can
 # check for a particular regression.
-slow = Trait(1, TraitRange(1, 3), TraitRange(-3, -1))
+slow = Trait(1, Range(1, 3), Range(-3, -1))
 try:
     del slow.handler.fast_validate
 except AttributeError:
@@ -1187,23 +937,14 @@ except AttributeError:
 
 
 class complex_value(HasTraits):
-    num1 = Trait(1, TraitRange(1, 5), TraitRange(-5, -1))
-    num2 = Trait(
-        1,
-        TraitRange(1, 5),
-        TraitPrefixList("one", "two", "three", "four", "five"),
-    )
-    num3 = Trait(
-        1,
-        TraitRange(1, 5),
-        TraitPrefixMap({"one": 1, "two": 2, "three": 3, "four": 4, "five": 5}),
-    )
+    num1 = Trait(1, Range(1, 5), Range(-5, -1))
     num4 = Trait(1, Trait(1, Tuple, slow), 10)
     num5 = Trait(1, 10, Trait(1, Tuple, slow))
 
 
 class test_complex_value(test_base2):
-    obj = complex_value()
+    def setUp(self):
+        self.obj = complex_value()
 
     def test_num1(self):
         self.check_values(
@@ -1240,123 +981,6 @@ class test_complex_value(test_base2):
         )
 
 
-class list_value(HasTraits):
-    # Trait definitions:
-    list1 = Trait([2], TraitList(Trait([1, 2, 3, 4]), maxlen=4))
-    list2 = Trait([2], TraitList(Trait([1, 2, 3, 4]), minlen=1, maxlen=4))
-    alist = List()
-
-
-class test_list_value(test_base2):
-
-    obj = list_value()
-
-    def setUp(self):
-        test_base2.setUp(self)
-        self.last_event = None
-
-    def tearDown(self):
-        del self.last_event
-
-    def del_range(self, list, index1, index2):
-        del list[index1:index2]
-
-    def del_extended_slice(self, list, index1, index2, step):
-        del list[index1:index2:step]
-
-    def check_list(self, list):
-        self.assertEqual(list, [2])
-        self.assertEqual(len(list), 1)
-        list.append(3)
-        self.assertEqual(len(list), 2)
-        list[1] = 2
-        self.assertEqual(list[1], 2)
-        self.assertEqual(len(list), 2)
-        list[0] = 1
-        self.assertEqual(list[0], 1)
-        self.assertEqual(len(list), 2)
-        self.assertRaises(TraitError, self.indexed_assign, list, 0, 5)
-        self.assertRaises(TraitError, list.append, 5)
-        self.assertRaises(TraitError, list.extend, [1, 2, 3])
-        list.extend([3, 4])
-        self.assertEqual(list, [1, 2, 3, 4])
-        self.assertRaises(TraitError, list.append, 1)
-        self.assertRaises(
-            ValueError, self.extended_slice_assign, list, 0, 4, 2, [4, 5, 6]
-        )
-        del list[1]
-        self.assertEqual(list, [1, 3, 4])
-        del list[0]
-        self.assertEqual(list, [3, 4])
-        list[:0] = [1, 2]
-        self.assertEqual(list, [1, 2, 3, 4])
-        self.assertRaises(
-            TraitError, self.indexed_range_assign, list, 0, 0, [1]
-        )
-        del list[0:3]
-        self.assertEqual(list, [4])
-        self.assertRaises(
-            TraitError, self.indexed_range_assign, list, 0, 0, [4, 5]
-        )
-
-    def test_list1(self):
-        self.check_list(self.obj.list1)
-
-    def test_list2(self):
-        self.check_list(self.obj.list2)
-        self.assertRaises(TraitError, self.del_range, self.obj.list2, 0, 1)
-        self.assertRaises(
-            TraitError, self.del_extended_slice, self.obj.list2, 4, -5, -1
-        )
-
-    def assertLastTraitListEventEqual(self, index, removed, added):
-        self.assertEqual(self.last_event.index, index)
-        self.assertEqual(self.last_event.removed, removed)
-        self.assertEqual(self.last_event.added, added)
-
-    def test_trait_list_event(self):
-        """ Record TraitListEvent behavior.
-        """
-        # FIXME: The behavior of TraitListEvent is suboptimal with
-        # respect to extended slice changes. Previously, TraitListObject
-        # used to have a __setitem__() and a separate __setslice__() to
-        # handle non-extended slices. Extended slices were added to the
-        # underlying list object later. The __setitem__() code handled
-        # the new extended slices, but created the TraitListEvent in the
-        # same way it did for an integer index; namely it wrapped the
-        # value with a list. For simple slices, the `index` attribute of
-        # the TraitListEvent is an integer, and the `added` list is just
-        # the list of values added. For an extended slice, the `index`
-        # attribute is the slice object and the `added` list is the list
-        # of values wrapped in another list.
-        self.obj.alist = [1, 2, 3, 4]
-        self.obj.on_trait_change(self._record_trait_list_event, "alist_items")
-        del self.obj.alist[0]
-        self.assertLastTraitListEventEqual(0, [1], [])
-        self.obj.alist.append(5)
-        self.assertLastTraitListEventEqual(3, [], [5])
-        self.obj.alist[0:2] = [6, 7]
-        self.assertLastTraitListEventEqual(0, [2, 3], [6, 7])
-        self.obj.alist[0:2:1] = [8, 9]
-        self.assertLastTraitListEventEqual(0, [6, 7], [8, 9])
-        old_event = self.last_event
-        self.obj.alist[0:2:1] = [8, 9]
-        # If no values changed, no new TraitListEvent will be generated.
-        self.assertIs(self.last_event, old_event)
-        self.obj.alist[0:4:2] = [10, 11]
-        self.assertLastTraitListEventEqual(
-            slice(0, 4, 2), [[8, 4]], [[10, 11]]
-        )
-        del self.obj.alist[1:4:2]
-        self.assertLastTraitListEventEqual(slice(1, 4, 2), [[9, 5]], [])
-        self.obj.alist = [1, 2, 3, 4]
-        del self.obj.alist[2:4]
-        self.assertLastTraitListEventEqual(2, [3, 4], [])
-
-    def _record_trait_list_event(self, object, name, old, new):
-        self.last_event = new
-
-
 class ThisDummy(HasTraits):
     allows_none = This()
     disallows_none = This(allow_none=False)
@@ -1385,3 +1009,65 @@ class TestThis(unittest.TestCase):
         with self.assertRaises(TraitError):
             d.allows_none = object()
         self.assertIsNone(d.allows_none)
+
+
+class ComparisonModeTests(unittest.TestCase):
+    def test_comparison_mode_none(self):
+        class HasComparisonMode(HasTraits):
+            bar = Trait(comparison_mode=ComparisonMode.none)
+
+        old_compare = HasComparisonMode()
+        events = []
+        old_compare.on_trait_change(lambda: events.append(None), "bar")
+
+        some_list = [1, 2, 3]
+
+        self.assertEqual(len(events), 0)
+        old_compare.bar = some_list
+        self.assertEqual(len(events), 1)
+        old_compare.bar = some_list
+        self.assertEqual(len(events), 2)
+        old_compare.bar = [1, 2, 3]
+        self.assertEqual(len(events), 3)
+        old_compare.bar = [4, 5, 6]
+        self.assertEqual(len(events), 4)
+
+    def test_comparison_mode_identity(self):
+        class HasComparisonMode(HasTraits):
+            bar = Trait(comparison_mode=ComparisonMode.identity)
+
+        old_compare = HasComparisonMode()
+        events = []
+        old_compare.on_trait_change(lambda: events.append(None), "bar")
+
+        some_list = [1, 2, 3]
+
+        self.assertEqual(len(events), 0)
+        old_compare.bar = some_list
+        self.assertEqual(len(events), 1)
+        old_compare.bar = some_list
+        self.assertEqual(len(events), 1)
+        old_compare.bar = [1, 2, 3]
+        self.assertEqual(len(events), 2)
+        old_compare.bar = [4, 5, 6]
+        self.assertEqual(len(events), 3)
+
+    def test_comparison_mode_equality(self):
+        class HasComparisonMode(HasTraits):
+            bar = Trait(comparison_mode=ComparisonMode.equality)
+
+        old_compare = HasComparisonMode()
+        events = []
+        old_compare.on_trait_change(lambda: events.append(None), "bar")
+
+        some_list = [1, 2, 3]
+
+        self.assertEqual(len(events), 0)
+        old_compare.bar = some_list
+        self.assertEqual(len(events), 1)
+        old_compare.bar = some_list
+        self.assertEqual(len(events), 1)
+        old_compare.bar = [1, 2, 3]
+        self.assertEqual(len(events), 1)
+        old_compare.bar = [4, 5, 6]
+        self.assertEqual(len(events), 2)
