@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2022 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2025 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -34,10 +34,8 @@ Visualization:
 """
 
 from types import FunctionType, MethodType
-import warnings
 
 from .constants import (
-    ComparisonMode,
     DefaultValue,
     TraitKind,
 )
@@ -73,7 +71,6 @@ from .trait_handlers import (
 from .trait_factory import (
     TraitFactory,
 )
-from .util.deprecated import deprecated
 
 # Constants
 
@@ -209,16 +206,16 @@ def Trait(*value_type, **metadata):
     |                   |   Trait(i)    |                                     |
     +-------------------+---------------+-------------------------------------+
     | Trait(*handler*)  | Trait(        | Assignment to this trait is         |
-    |                   | TraitEnum )   | validated by an object derived from |
+    |                   | TraitEnum)    | validated by an object derived from |
     |                   |               | **traits.TraitHandler**.            |
     +-------------------+---------------+-------------------------------------+
     | Trait(*default*,  | Trait(0.0, 0.0| This is the most general form of    |
-    | { *type* |        | 'stuff',      | the function. The notation:         |
+    | {*type* |         | 'stuff',      | the function. The notation:         |
     | *constant* |      | TupleType)    | ``{...|...|...}+`` means a list of  |
     | *dict* | *class* ||               | one or more of any of the items     |
     | *function* |      |               | listed between the braces. Thus, the|
     | *handler* |       |               | most general form of the function   |
-    | *trait* }+ )      |               | consists of a default value,        |
+    | *trait*}+)        |               | consists of a default value,        |
     |                   |               | followed by one or more of several  |
     |                   |               | possible items. A trait defined by  |
     |                   |               | multiple items is called a          |
@@ -478,21 +475,6 @@ class _TraitMaker(object):
                 trait.post_setattr = post_setattr
                 trait.is_mapped = handler.is_mapped
 
-        rich_compare = metadata.get("rich_compare")
-        if rich_compare is not None:
-            # Ref: enthought/traits#602
-            warnings.warn(
-                "The 'rich_compare' metadata has been deprecated. Please "
-                "use the 'comparison_mode' metadata instead. In a future "
-                "release, rich_compare will have no effect.",
-                DeprecationWarning,
-                stacklevel=4,
-            )
-            if rich_compare:
-                trait.comparison_mode = ComparisonMode.equality
-            else:
-                trait.comparison_mode = ComparisonMode.identity
-
         comparison_mode = metadata.pop("comparison_mode", None)
         if comparison_mode is not None:
             trait.comparison_mode = comparison_mode
@@ -652,57 +634,3 @@ class ForwardProperty(object):
 
 # Generic trait with 'object' behavior:
 generic_trait = CTrait(TraitKind.generic)
-
-
-# User interface related color and font traits
-
-@deprecated("'Color' in 'traits' package has been deprecated. "
-            "Use 'Color' from 'traitsui' package instead.")
-def Color(*args, **metadata):
-    """ Returns a trait whose value must be a GUI toolkit-specific color.
-
-    .. deprecated:: 6.1.0
-        ``Color`` trait in this package will be removed in the future. It is
-        replaced by ``Color`` trait in TraitsUI package.
-    """
-    from traitsui.toolkit_traits import ColorTrait
-
-    return ColorTrait(*args, **metadata)
-
-
-Color = TraitFactory(Color)
-
-
-@deprecated("'RGBColor' in 'traits' package has been deprecated. "
-            "Use 'RGBColor' from 'traitsui' package instead.")
-def RGBColor(*args, **metadata):
-    """ Returns a trait whose value must be a GUI toolkit-specific RGB-based
-    color.
-
-    .. deprecated:: 6.1.0
-        ``RGBColor`` trait in this package will be removed in the future. It is
-        replaced by ``RGBColor`` trait in TraitsUI package.
-    """
-    from traitsui.toolkit_traits import RGBColorTrait
-
-    return RGBColorTrait(*args, **metadata)
-
-
-RGBColor = TraitFactory(RGBColor)
-
-
-@deprecated("'Font' in 'traits' package has been deprecated. "
-            "Use 'Font' from 'traitsui' package instead.")
-def Font(*args, **metadata):
-    """ Returns a trait whose value must be a GUI toolkit-specific font.
-
-    .. deprecated:: 6.1.0
-        ``Font`` trait in this package will be removed in the future. It is
-        replaced by ``Font`` trait in TraitsUI package.
-    """
-    from traitsui.toolkit_traits import FontTrait
-
-    return FontTrait(*args, **metadata)
-
-
-Font = TraitFactory(Font)
